@@ -39,8 +39,14 @@ extern "C" {
  * the \ref ioctls through type-safe interfaces and provides functions to change
  * the appearance of the device.
  *
+ * Development
+ * ===========
+ * The git repository is available here:
+ *
+ *   - https://gitlab.freedesktop.org/libevdev/libevdev
+ *
  * Development of libevdev is discussed on
- * [input-tools@lists.freedesktop.org](http://lists.freedesktop.org/mailman/listinfo/input-tools)
+ * [input-tools@lists.freedesktop.org](http://lists.freedesktop.org/mailman/listinfo/input-tools).
  * Please submit patches, questions or general comments there.
  *
  * Handling events and SYN_DROPPED
@@ -95,9 +101,10 @@ extern "C" {
  *
  *      kernel → libevdev → xf86-input-evdev → X server → X client
  *
- * For Weston/Wayland, the stack would look like this:
+ * For anything using libinput (e.g. most Wayland compositors), the stack
+ * the stack would look like this:
  *
- *      kernel → libevdev → Weston → Wayland client
+ *      kernel → libevdev → libinput → Compositor → Wayland client
  *
  * libevdev does **not** have knowledge of X clients or Wayland clients, it is
  * too low in the stack.
@@ -142,7 +149,7 @@ extern "C" {
  * @endcode
  *
  * A more complete example is available with the libevdev-events tool here:
- * http://cgit.freedesktop.org/libevdev/tree/tools/libevdev-events.c
+ * https://gitlab.freedesktop.org/libevdev/libevdev/blob/master/tools/libevdev-events.c
  *
  * Backwards compatibility with older kernel
  * =========================================
@@ -155,10 +162,14 @@ extern "C" {
  * libevdev is licensed under the
  * [X11 license](http://cgit.freedesktop.org/libevdev/tree/COPYING).
  *
+ * Bindings
+ * ===================
+ * - Python: https://gitlab.freedesktop.org/libevdev/python-libevdev
+ *
  * Reporting bugs
  * ==============
- * Please report bugs in the freedesktop.org bugzilla under the libevdev product:
- * https://bugs.freedesktop.org/enter_bug.cgi?product=libevdev
+ * Please report bugs in the freedesktop.org GitLab instance:
+ * https://gitlab.freedesktop.org/libevdev/libevdev/issues/
  */
 
 /**
@@ -655,9 +666,9 @@ extern "C" {
 /**
  * @defgroup bits Querying device capabilities
  *
- * Abstraction functions to handle device capabilities, specificially
+ * Abstraction functions to handle device capabilities, specifically
  * device properties such as the name of the device and the bits
- * representing the events suppported by this device.
+ * representing the events supported by this device.
  *
  * The logical state returned may lag behind the physical state of the device.
  * libevdev queries the device state on libevdev_set_fd() and then relies on
@@ -681,14 +692,14 @@ extern "C" {
  * The Linux kernel requires all axes on a device to have a semantic
  * meaning, matching the axis names in linux/input.h. Some devices merely
  * export a number of axes beyond the available axis list. For those
- * devices, the multitouch information is invalid. Specfically, if a device
- * provides the ABS_MT_SLOT axis AND also the (ABS_MT_SLOT - 1) axis, the
+ * devices, the multitouch information is invalid. Specifically, if a device
+ * provides the ABS_MT_SLOT axis AND also the ABS_RESERVED axis, the
  * device is not treated as multitouch device. No slot information is
  * available and the ABS_MT axis range for these devices is treated as all
  * other EV_ABS axes.
  *
  * Note that because of limitations in the kernel API, such fake multitouch
- * devices can not be reliably synched after a SYN_DROPPED event. libevdev
+ * devices can not be reliably synced after a SYN_DROPPED event. libevdev
  * ignores all ABS_MT axis values during the sync process and instead
  * relies on the device to send the current axis value with the first event
  * after SYN_DROPPED.
@@ -956,10 +967,14 @@ enum libevdev_grab_mode {
  * Grabbing an already grabbed device, or ungrabbing an ungrabbed device is
  * a noop and always succeeds.
  *
+ * A grab is an operation tied to a file descriptor, not a device. If a
+ * client changes the file descriptor with libevdev_change_fd(), it must
+ * also re-issue a grab with libevdev_grab().
+ *
  * @param dev The evdev device, already initialized with libevdev_set_fd()
  * @param grab If true, grab the device. Otherwise ungrab the device.
  *
- * @return 0 if the device was successfull grabbed or ungrabbed, or a
+ * @return 0 if the device was successfully grabbed or ungrabbed, or a
  * negative errno in case of failure.
  */
 int libevdev_grab(struct libevdev *dev, enum libevdev_grab_mode grab);
@@ -985,7 +1000,7 @@ int libevdev_grab(struct libevdev *dev, enum libevdev_grab_mode grab);
  * events like EV_REL.
  *
  * Unless otherwise specified, libevdev function behavior is undefined until
- * a successfull call to libevdev_set_fd().
+ * a successful call to libevdev_set_fd().
  *
  * @param dev The evdev device
  * @param fd The file descriptor for the device
@@ -1024,6 +1039,9 @@ int libevdev_set_fd(struct libevdev* dev, int fd);
  *
  * The fd may be open in O_RDONLY or O_RDWR.
  *
+ * After changing the fd, the device is assumed ungrabbed and a caller must
+ * call libevdev_grab() again.
+ *
  * It is an error to call this function before calling libevdev_set_fd().
  *
  * @param dev The evdev device, already initialized with libevdev_set_fd()
@@ -1518,9 +1536,12 @@ int libevdev_get_event_value(const struct libevdev *dev, unsigned int type, unsi
  * @param value The new value to set
  *
  * @return 0 on success, or -1 on failure.
- * @retval -1 the device does not have the event type or code enabled, or the code is outside the
- * allowed limits for the given type, or the type cannot be set, or the
- * value is not permitted for the given code.
+ * @retval -1
+ * - the device does not have the event type or
+ * - code enabled, or the code is outside the, or
+ * - the code is outside the allowed limits for the given type, or
+ * - the type cannot be set, or
+ * - the value is not permitted for the given code.
  *
  * @see libevdev_set_slot_value
  * @see libevdev_get_event_value
@@ -1595,9 +1616,11 @@ int libevdev_get_slot_value(const struct libevdev *dev, unsigned int slot, unsig
  * @param value The new value to set
  *
  * @return 0 on success, or -1 on failure.
- * @retval -1 the device does not have the event code enabled, or the code is
- * outside the allowed limits for multitouch events, or the slot number is outside
- * the limits for this device, or the device does not support multitouch events.
+ * @retval -1
+ * - the device does not have the event code enabled, or
+ * - the code is outside the allowed limits for multitouch events, or
+ * - the slot number is outside the limits for this device, or
+ * - the device does not support multitouch events.
  *
  * @see libevdev_set_event_value
  * @see libevdev_get_slot_value
@@ -2006,6 +2029,29 @@ const char * libevdev_event_type_get_name(unsigned int type);
  */
 const char * libevdev_event_code_get_name(unsigned int type, unsigned int code);
 
+/**
+ * @ingroup misc
+ *
+ * This function resolves the event value for a code.
+ *
+ * For almost all event codes this will return NULL as the value is just a
+ * numerical value. As of kernel 4.17, the only event code that will return
+ * a non-NULL value is EV_ABS/ABS_MT_TOOL_TYPE.
+ *
+ * @param type The event type for the value to query (EV_ABS, etc.)
+ * @param code The event code for the value to query (e.g. ABS_MT_TOOL_TYPE)
+ * @param value The event value to return the name for (e.g. MT_TOOL_PALM)
+ *
+ * @return The name of the given event value (e.g. MT_TOOL_PALM) or NULL for
+ * an invalid type or code or NULL for an axis that has numerical values
+ * only.
+ *
+ * @note The list of names is compiled into libevdev. If the kernel adds new
+ * defines for new event values, libevdev will not automatically pick these up.
+ */
+const char * libevdev_event_value_get_name(unsigned int type,
+					   unsigned int code,
+					   int value);
 /**
  * @ingroup misc
  *
@@ -2114,6 +2160,136 @@ int libevdev_event_code_from_name(unsigned int type, const char *name);
 int libevdev_event_code_from_name_n(unsigned int type, const char *name,
 				    size_t len);
 
+/**
+ * @ingroup misc
+ *
+ * Look up an event value by its type, code and name. Event values start
+ * with a fixed prefix followed by their name (eg., "MT_TOOL_PALM"). The
+ * prefix must be included in the name. It returns the constant assigned
+ * to the event code or -1 if not found.
+ *
+ * You have to pass the event type and code where to look for the name. For
+ * instance, to resolve "MT_TOOL_PALM" you need to pass EV_ABS as type,
+ * ABS_MT_TOOL_TYPE as code and "MT_TOOL_PALM" as string.
+ *
+ * As of kernel 4.17, only EV_ABS/ABS_MT_TOOL_TYPE support name resolution.
+ *
+ * @param type The event type (EV_* constant) where to look for the name.
+ * @param code The event code (ABS_* constant) where to look for the name.
+ * @param name A non-NULL string describing an input-event value
+ * ("MT_TOOL_TYPE", ...)
+ *
+ * @return The given value constant for the name or -1 if not found.
+ */
+int libevdev_event_value_from_name(unsigned int type, unsigned int code,
+				   const char *name);
+
+/**
+ * @ingroup misc
+ *
+ * Look up an event type for a  event code name. For example, the name
+ * "ABS_Y" returns EV_ABS. For the lookup to succeed, the name must be
+ * unique, which is the case for all defines as of kernel 5.0 and likely to
+ * be the case in the future.
+ *
+ * This is equivalent to libevdev_event_type_from_name() but takes the code
+ * name instead of the type name.
+ *
+ * @param name A non-NULL string describing an input-event value
+ * ("ABS_X", "REL_Y", "KEY_A", ...)
+ *
+ * @return The given event code for the name or -1 if not found.
+ */
+int
+libevdev_event_type_from_code_name(const char *name);
+
+/**
+ * @ingroup misc
+ *
+ * Look up an event type for a  event code name. For example, the name
+ * "ABS_Y" returns EV_ABS. For the lookup to succeed, the name must be
+ * unique, which is the case for all defines as of kernel 5.0 and likely to
+ * be the case in the future.
+ *
+ * This is equivalent to libevdev_event_type_from_name_n() but takes the code
+ * name instead of the type name.
+ *
+ * @param name A non-NULL string describing an input-event value
+ * ("ABS_X", "REL_Y", "KEY_A", ...)
+ * @param len The length of the passed string excluding any terminating 0
+ * character.
+ *
+ * @return The given event code for the name or -1 if not found.
+ */
+int
+libevdev_event_type_from_code_name_n(const char *name, size_t len);
+
+/**
+ * @ingroup misc
+ *
+ * Look up an event code by its name. For example, the name "ABS_Y"
+ * returns 1. For the lookup to succeed, the name must be unique, which is
+ * the case for all defines as of kernel 5.0 and likely to be the case in
+ * the future.
+ *
+ * This is equivalent to libevdev_event_code_from_name() without the need
+ * for knowing the event type.
+ *
+ * @param name A non-NULL string describing an input-event value
+ * ("ABS_X", "REL_Y", "KEY_A", ...)
+ *
+ * @return The given event code for the name or -1 if not found.
+ */
+int
+libevdev_event_code_from_code_name(const char *name);
+
+/**
+ * @ingroup misc
+ *
+ * Look up an event code by its name. For example, the name "ABS_Y"
+ * returns 1. For the lookup to succeed, the name must be unique, which is
+ * the case for all defines as of kernel 5.0 and likely to be the case in
+ * the future.
+ *
+ * This is equivalent to libevdev_event_code_from_name_n() without the need
+ * for knowing the event type.
+ *
+ * @param name A non-NULL string describing an input-event value
+ * ("ABS_X", "REL_Y", "KEY_A", ...)
+ * @param len The length of the passed string excluding any terminating 0
+ * character.
+ *
+ * @return The given event code for the name or -1 if not found.
+ */
+int
+libevdev_event_code_from_code_name_n(const char *name, size_t len);
+
+/**
+ * @ingroup misc
+ *
+ * Look up an event value by its type, code and name. Event values start
+ * with a fixed prefix followed by their name (eg., "MT_TOOL_PALM"). The
+ * prefix must be included in the name. It returns the constant assigned
+ * to the event code or -1 if not found.
+ *
+ * You have to pass the event type and code where to look for the name. For
+ * instance, to resolve "MT_TOOL_PALM" you need to pass EV_ABS as type,
+ * ABS_MT_TOOL_TYPE as code and "MT_TOOL_PALM" as string.
+ *
+ * As of kernel 4.17, only EV_ABS/ABS_MT_TOOL_TYPE support name resolution.
+ *
+ * @param type The event type (EV_* constant) where to look for the name.
+ * @param code The event code (ABS_* constant) where to look for the name.
+ * @param name A non-NULL string describing an input-event value
+ * ("MT_TOOL_TYPE", ...)
+ * @param len The length of the string in @p name excluding any terminating 0
+ * character.
+ *
+ * @return The given value constant for the name or -1 if not found.
+ */
+int libevdev_event_value_from_name_n(unsigned int type, unsigned int code,
+				     const char *name, size_t len);
+
 /**
  * @ingroup misc
  *
